import * as Leaf from "leaf-ts"
import { Session } from "./session"
import * as pathModule from "path"
export class Action {
    static actionsFromScriptRecursive(path: string): ActionData[] {
        if (path.slice(-8) !== ".flow.js") {
            return []
        }
        let base = pathModule.basename(path, ".flow.js")
        let dir = pathModule.dirname(path)
        let obj = require(path)
        let actions = obj.actions || []
        let after = []
        if (obj.after) {
            for (let item of obj.after) {
                after = after.concat(this.actionsFromScriptRecursive(pathModule.join(dir, item) + ".flow.js"))
            }
        }
        return after.concat(actions)
    }
    constructor(public session: Session, public readonly info: ActionData) {
    }
    go(callback: (err?: Error, result?: any) => void) {
        let data = this.session.build(this.info.data)
        let fn = this.session.getAPI(this.info.name)
        if (!fn) {
            callback(new Error("Unknown method:" + this.info.name))
            return
        }
        fn(Leaf.Util.clone(data), (err, result) => {
            if (this.session.verbose) {
                console.log(this.info.name, ":", data, "==>", err, result)
            }
            if (err) {
                callback(err)
                return
            }
            try {
                this.should(result)
                this.check(result)
                this.solveResult(result)
            } catch (e) {
                callback(e)
                return
            }
            callback(null, result)
        })
        // 1. build data by session.build(info.data)
        // 2. send request
        // ==========after return=========
        // 3. check by info.shouldBe and info.check
        // 4. save by info.save
        // 5. saveHeader by info.saveHeader
    }

    check(data, check = this.info.check, key = "<Response>") {
        if (typeof check == "undefined" || check === null) {
            return
        }
        if (typeof check == "function") {
            return check(data)
        } else if (typeof check == "boolean") {
            if (check && !data) {
                throw new Error(`"${key}" should exists`)
            } else if (!check && data) {
                throw new Error(`"${key}" shouldn't exists`)
            }
        } else {
            // @ts-ignore
            for (let path in check) {
                let check = this.info.check[path]
                if (check === null || typeof check == "undefined") continue
                let value = Leaf.Util.getObjectValue(data, path)
                this.check(value, check, path)
            }
        }
    }
    should(data) {
        for (let path in this.info.shouldBe || {}) {
            let value = Leaf.Util.getObjectValue(data, path)
            let valueShould = this.info.shouldBe[path]
            if (!Leaf.Util.deepEqual(value, valueShould)) {
                throw new Error(`${path} should be ${valueShould} but is ${value}`)
            }
        }
    }
    solveResult(data: any) {
        // 1. save $name into session
        // 2. saveHeader into session
        if (typeof this.info.save === "string") {
            this.session.save(this.info.save, Leaf.Util.clone(data))
        } else {
            for (let savePath in this.info.save || {}) {
                let saveName = this.info.save[savePath]
                let v = Leaf.Util.getObjectValue(data, savePath)
                this.session.save(saveName, v)
            }
        }
        if (typeof this.info.saveHeader === "string") {
            this.session.saveHeader(this.info.saveHeader, Leaf.Util.clone(data))
        } else {
            for (let saveHeaderPath in this.info.saveHeader || {}) {
                let saveHeaderName = this.info.saveHeader[saveHeaderPath]
                let v = Leaf.Util.getObjectValue(data, saveHeaderPath)
                this.session.saveHeader(saveHeaderName, v)
            }
        }
    }
}


export interface ActionData {
    name: string
    data?: any
    shouldBe?: {
        [path: string]: any
    }
    example?: any
    save?: {
        [path: string]: string
    } | string
    saveHeader?: {
        [path: string]: string
    }
    check?: Checker
}

export type Checker = ((v: any) => boolean) | boolean | {
    [key: string]: Checker
}

