"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Session {
    constructor(base, APIContents) {
        this.base = base;
        this.APIContents = APIContents;
        this.verbose = false;
        this.datas = {};
        this.headers = {};
        this.apis = {};
        this.generateAPIs();
    }
    generateAPIs() {
        this.apis = {};
        for (let content of this.APIContents) {
            let APIs = content;
            for (let name in APIs) {
                if (name.indexOf("Generate") === 0 && name !== "GenerateAPI") {
                    let start = "Generate".length;
                    let subName = name.slice(start);
                    let Cons = APIs[name]();
                    let apiService = new Cons();
                    this.apis[subName] = apiService;
                    let factory = apiService.factory;
                    factory.option.headers = this.headers;
                    factory.option.root = this.base;
                }
            }
        }
    }
    getAPI(name) {
        let sName, aName;
        if (name.indexOf(".") >= 0) {
            [sName, aName] = name.split(".");
        }
        else {
            aName = name;
        }
        for (let s in this.apis) {
            if (!sName || s === sName) {
                for (let a in this.apis[s]) {
                    if (a === aName) {
                        return this.apis[s][a].bind(this.apis[s]);
                    }
                }
            }
        }
        return null;
    }
    // save/get/build datas
    save(name, value) {
        this.datas[name] = value;
    }
    get(name) {
        return this.datas[name];
    }
    build(data) {
        if (!data) {
            return data;
        }
        if (typeof data === "string") {
            return this.evalString(data);
        }
        if (data instanceof Array) {
            return data.map(item => this.build(item));
        }
        if (typeof data === "object") {
            let result = {};
            for (let prop in data) {
                result[prop] = this.build(data[prop]);
            }
            return result;
        }
        return data;
    }
    // save and update headers
    saveHeader(name, value) {
        this.headers[name] = value;
    }
    // reset saved datas/headers
    clear() {
        this.datas = {};
        for (let prop in this.headers) {
            delete this.headers[prop];
        }
    }
    evalString(data) {
        if (!data)
            return data;
        if (data[0] == "$") {
            return this.datas[data.slice(1)] || data;
        }
        return data;
    }
}
exports.Session = Session;
//# sourceMappingURL=session.js.map